@@ -15,6 +15,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, see <https://www.gnu.org/licenses/>.
+ * SPDX-License-Identifier: GPL-3.0-or-later
  */
 
 #include <config.h>
@@ -25,7 +26,6 @@
 #include <stdarg.h>
 #include <string.h>
 #include <errno.h>
-#include <assert.h>
 #include <sys/stat.h>
 #include <unistd.h>
 #ifdef HAVE_LOCALE_H
@@ -97,13 +97,12 @@ static const char *opt_passphrase;
 static char *opt_prompt;
 static int opt_status_msg;
 static const char *opt_agent_program;
-static int opt_debug_use_ocb;
 
 static char *get_passphrase (int promptno);
 static void release_passphrase (char *pw);
 
 
-static ARGPARSE_OPTS opts[] = {
+static gpgrt_opt_t opts[] = {
   ARGPARSE_group (300, N_("@Commands:\n ")),
 
   ARGPARSE_c (oProtect,   "protect",   "protect a private key"),
@@ -145,9 +144,11 @@ my_strusage (int level)
   const char *p;
   switch (level)
     {
+    case  9: p = "GPL-3.0-or-later"; break;
     case 11: p = "gpg-protect-tool (" GNUPG_NAME ")";
       break;
     case 13: p = VERSION; break;
+    case 14: p = GNUPG_DEF_COPYRIGHT_LINE; break;
     case 17: p = PRINTABLE_OS_NAME; break;
     case 19: p = _("Please report bugs to <@EMAIL@>.\n"); break;
 
@@ -199,10 +200,10 @@ make_canonical (const char *fname, const char *buf, size_t buflen)
       return NULL;
     }
   len = gcry_sexp_sprint (sexp, GCRYSEXP_FMT_CANON, NULL, 0);
-  assert (len);
+  log_assert (len);
   result = xmalloc (len);
   len = gcry_sexp_sprint (sexp, GCRYSEXP_FMT_CANON, result, len);
-  assert (len);
+  log_assert (len);
   gcry_sexp_release (sexp);
   return result;
 }
@@ -223,10 +224,10 @@ make_advanced (const unsigned char *buf, size_t buflen)
       return NULL;
     }
   len = gcry_sexp_sprint (sexp, GCRYSEXP_FMT_ADVANCED, NULL, 0);
-  assert (len);
+  log_assert (len);
   result = xmalloc (len);
   len = gcry_sexp_sprint (sexp, GCRYSEXP_FMT_ADVANCED, result, len);
-  assert (len);
+  log_assert (len);
   gcry_sexp_release (sexp);
   return result;
 }
@@ -235,7 +236,7 @@ make_advanced (const unsigned char *buf, size_t buflen)
 static char *
 read_file (const char *fname, size_t *r_length)
 {
-  FILE *fp;
+  estream_t fp;
   char *buf;
   size_t buflen;
 
@@ -243,10 +244,8 @@ read_file (const char *fname, size_t *r_length)
     {
       size_t nread, bufsize = 0;
 
-      fp = stdin;
-#ifdef HAVE_DOSISH_SYSTEM
-      setmode ( fileno(fp) , O_BINARY );
-#endif
+      fp = es_stdin;
+      es_set_binary (fp);
       buf = NULL;
       buflen = 0;
 #define NCHUNK 8192
@@ -258,8 +257,8 @@ read_file (const char *fname, size_t *r_length)
           else
             buf = xrealloc (buf, bufsize);
 
-          nread = fread (buf+buflen, 1, NCHUNK, fp);
-          if (nread < NCHUNK && ferror (fp))
+          nread = es_fread (buf+buflen, 1, NCHUNK, fp);
+          if (nread < NCHUNK && es_ferror (fp))
             {
               log_error ("error reading '[stdin]': %s\n", strerror (errno));
               xfree (buf);
@@ -275,30 +274,30 @@ read_file (const char *fname, size_t *r_length)
     {
       struct stat st;
 
-      fp = fopen (fname, "rb");
+      fp = es_fopen (fname, "rb");
       if (!fp)
         {
           log_error ("can't open '%s': %s\n", fname, strerror (errno));
           return NULL;
         }
 
-      if (fstat (fileno(fp), &st))
+      if (fstat (es_fileno (fp), &st))
         {
           log_error ("can't stat '%s': %s\n", fname, strerror (errno));
-          fclose (fp);
+          es_fclose (fp);
           return NULL;
         }
 
       buflen = st.st_size;
       buf = xmalloc (buflen+1);
-      if (fread (buf, buflen, 1, fp) != 1)
+      if (es_fread (buf, buflen, 1, fp) != 1)
         {
           log_error ("error reading '%s': %s\n", fname, strerror (errno));
-          fclose (fp);
+          es_fclose (fp);
           xfree (buf);
           return NULL;
         }
-      fclose (fp);
+      es_fclose (fp);
     }
 
   *r_length = buflen;
@@ -319,6 +318,7 @@ read_key (const char *fname)
   if (buflen >= 4 && !memcmp (buf, "Key:", 4))
     {
       log_error ("Extended key format is not supported by this tool\n");
+      xfree (buf);
       return NULL;
     }
   key = make_canonical (fname, buf, buflen);
@@ -342,8 +342,7 @@ read_and_protect (const char *fname)
     return;
 
   pw = get_passphrase (1);
-  rc = agent_protect (key, pw, &result, &resultlen, 0,
-                      opt_debug_use_ocb? 1 : -1);
+  rc = agent_protect (key, pw, &result, &resultlen, 0);
   release_passphrase (pw);
   xfree (key);
   if (rc)
@@ -370,7 +369,7 @@ read_and_protect (const char *fname)
 static void
 read_and_unprotect (ctrl_t ctrl, const char *fname)
 {
-  int  rc;
+  gpg_error_t err;
   unsigned char *key;
   unsigned char *result;
   size_t resultlen;
@@ -381,15 +380,15 @@ read_and_unprotect (ctrl_t ctrl, const char *fname)
   if (!key)
     return;
 
-  rc = agent_unprotect (ctrl, key, (pw=get_passphrase (1)),
-                        protected_at, &result, &resultlen);
+  err = agent_unprotect (ctrl, key, (pw=get_passphrase (1)),
+                         protected_at, &result, &resultlen);
   release_passphrase (pw);
   xfree (key);
-  if (rc)
+  if (err)
     {
       if (opt_status_msg)
         log_info ("[PROTECT-TOOL:] bad-passphrase\n");
-      log_error ("unprotecting the key failed: %s\n", gpg_strerror (rc));
+      log_error ("unprotecting the key failed: %s\n", gpg_strerror (err));
       return;
     }
   if (opt.verbose)
@@ -402,6 +401,12 @@ read_and_unprotect (ctrl_t ctrl, const char *fname)
         log_info ("key protection done at [unknown]\n");
     }
 
+  err = fixup_when_ecc_private_key (result, &resultlen);
+  if (err)
+    {
+      log_error ("malformed key: %s\n", gpg_strerror (err));
+      return;
+    }
   if (opt_armor)
     {
       char *p = make_advanced (result, resultlen);
@@ -439,7 +444,7 @@ read_and_shadow (const char *fname)
       return;
     }
   resultlen = gcry_sexp_canon_len (result, 0, NULL,NULL);
-  assert (resultlen);
+  log_assert (resultlen);
 
   if (opt_armor)
     {
@@ -475,7 +480,7 @@ show_shadow_info (const char *fname)
       return;
     }
   infolen = gcry_sexp_canon_len (info, 0, NULL,NULL);
-  assert (infolen);
+  log_assert (infolen);
 
   if (opt_armor)
     {
@@ -502,7 +507,7 @@ show_file (const char *fname)
     return;
 
   keylen = gcry_sexp_canon_len (key, 0, NULL,NULL);
-  assert (keylen);
+  log_assert (keylen);
 
   if (opt_canonical)
     {
@@ -558,13 +563,13 @@ show_keygrip (const char *fname)
 int
 main (int argc, char **argv )
 {
-  ARGPARSE_ARGS pargs;
+  gpgrt_argparse_t pargs;
   int cmd = 0;
   const char *fname;
   ctrl_t ctrl;
 
   early_system_init ();
-  set_strusage (my_strusage);
+  gpgrt_set_strusage (my_strusage);
   gcry_control (GCRYCTL_SUSPEND_SECMEM_WARN);
   log_set_prefix ("gpg-protect-tool", GPGRT_LOG_WITH_PREFIX);
 
@@ -577,8 +582,8 @@ main (int argc, char **argv )
 
   pargs.argc = &argc;
   pargs.argv = &argv;
-  pargs.flags=  1;  /* (do not remove the args) */
-  while (arg_parse (&pargs, opts) )
+  pargs.flags= ARGPARSE_FLAG_KEEP;
+  while (gpgrt_argparse (NULL, &pargs, opts))
     {
       switch (pargs.r_opt)
         {
@@ -603,11 +608,13 @@ main (int argc, char **argv )
         case oHaveCert: opt_have_cert = 1; break;
         case oPrompt: opt_prompt = pargs.r.ret_str; break;
         case oStatusMsg: opt_status_msg = 1; break;
-        case oDebugUseOCB: opt_debug_use_ocb = 1; break;
+        case oDebugUseOCB: /* dummy */; break;
 
         default: pargs.err = ARGPARSE_PRINT_ERROR; break;
 	}
     }
+  gpgrt_argparse (NULL, &pargs, NULL);  /* Release internal state.  */
+
   if (log_get_errorcount (0))
     exit (2);
 
@@ -615,7 +622,7 @@ main (int argc, char **argv )
   if (argc == 1)
     fname = *argv;
   else if (argc > 1)
-    usage (1);
+    gpgrt_usage (1);
 
   /* Allocate an CTRL object.  An empty object should be sufficient.  */
   ctrl = xtrycalloc (1, sizeof *ctrl);
@@ -729,7 +736,7 @@ get_passphrase (int promptno)
                    gpg_strerror (err));
       agent_exit (0);
     }
-  assert (pw);
+  log_assert (pw);
 
   return pw;
 }
@@ -785,7 +792,10 @@ agent_askpin (ctrl_t ctrl,
   passphrase = get_passphrase (0);
   size = strlen (passphrase);
   if (size >= pininfo->max_length)
-    return gpg_error (GPG_ERR_TOO_LARGE);
+    {
+      xfree (passphrase);
+      return gpg_error (GPG_ERR_TOO_LARGE);
+    }
 
   memcpy (&pininfo->pin, passphrase, size);
   xfree (passphrase);
@@ -806,12 +816,15 @@ agent_askpin (ctrl_t ctrl,
 int
 agent_write_private_key (const unsigned char *grip,
                          const void *buffer, size_t length, int force,
+                         const char *serialno, const char *keyref,
                          time_t timestamp)
 {
   char hexgrip[40+4+1];
   char *p;
 
   (void)force;
+  (void)serialno;
+  (void)keyref;
   (void)timestamp;
 
   bin2hex (grip, 20, hexgrip);
