@@ -538,21 +538,32 @@ nve_t
 nvc_first (nvc_t pk)
 {
   nve_t entry;
+
+  if (!pk)
+    return NULL;
+
   for (entry = pk->first; entry; entry = entry->next)
     if (entry->name)
       return entry;
+
   return NULL;
 }
 
 
-/* Get the first entry with the given name.  */
+/* Get the first entry with the given name.  Return NULL if it does
+ * not exist.  */
 nve_t
 nvc_lookup (nvc_t pk, const char *name)
 {
   nve_t entry;
+
+  if (!pk)
+    return NULL;
+
   for (entry = pk->first; entry; entry = entry->next)
     if (entry->name && ascii_strcasecmp (entry->name, name) == 0)
       return entry;
+
   return NULL;
 }
 
@@ -597,6 +608,32 @@ nvc_get_string (nvc_t nvc, const char *name)
 }
 
 
+/* Return true (ie. a non-zero value) if NAME exists and its value is
+ * true; that is either "yes", "true", or a decimal value unequal to 0.  */
+int
+nvc_get_boolean (nvc_t nvc, const char *name)
+{
+  nve_t item;
+  const char *s;
+  int n;
+
+  if (!nvc)
+    return 0;
+  item = nvc_lookup (nvc, name);
+  if (!item)
+    return 0;
+  s = nve_value (item);
+  if (!s)
+    return 0;
+  n = atoi (s);
+  if (n)
+    return n;
+  if (!ascii_strcasecmp (s, "yes") || !ascii_strcasecmp (s, "true"))
+    return 1;
+  return 0;
+}
+
+
 
 
 /* Private key handling.  */
